import * as mysql2_promise from 'mysql2/promise';
import { Pool } from 'mysql2/promise';

interface Field {
    name: string;
    type: string;
    defaultValue?: string;
    length?: number;
    options?: string[];
    foreing?: {
        table: string;
        column: string;
    };
}

/**
 * Main class to handle MySQL database connections and queries.
 * Implements the Singleton pattern to ensure a single instance of the connection pool.
 */
declare class MySQL {
    private static instance;
    private pool;
    constructor();
    /**
     * Creates and returns a new instance of `TableQuery` for the specified table.
     * This method is used to start building queries for a specific table.
     *
     * @param {string} tableName - The name of the table to query.
     * @returns {TableQuery} - Returns a new instance of `TableQuery` for the specified table.
     *
     * @example
     * const usersTable = db.table('users');
     * const users = await usersTable.select(['id', 'name']).get();
     * console.log(users); // [{ id: 1, name: 'John' }, { id: 2, name: 'Jane' }]
     */
    table(tableName: string): TableQuery;
    /**
     * Executes a SQL query on the currently set database.
     *
     * @param {string} sqlQuery - The SQL query to execute.
     * @returns {Promise<{ status: string, message: string, data: any | null }>} - Returns a JSON object with the status, message, and data (if any).
     *
     * @example
     * const result = await db.query('SELECT * FROM users;');
     * console.log(result);
     * // { status: 'success', message: 'Query executed successfully', data: [{ id: 1, name: 'John' }, { id: 2, name: 'Jane' }] }
     *
     * @example
     * const result = await db.query('INVALID SQL QUERY;');
     * console.log(result);
     * // { status: 'error', message: 'SQL syntax error', data: null }
     */
    query(sqlQuery: string): Promise<{
        status: string;
        message: string;
        data: any | null;
    }>;
}
/**
 * Class to build and execute SQL queries for a specific table.
 * Supports operations like SELECT, INSERT, UPDATE, DELETE, and more.
 */
declare class TableQuery {
    #private;
    private conection;
    private nextType;
    private joins;
    private _orderBy;
    private _distinct;
    private _groupBy;
    private tableName;
    private fields;
    private query;
    private conditions;
    private limitValue;
    private pageValue;
    constructor(tableName: string, conection?: Pool | null);
    columns(): Columns;
    create(fields: Field[]): Promise<boolean>;
    drop(): Promise<boolean>;
    /**
     * Specifies the columns to select in a SELECT query.
     *
     * @param {string[]} fields - Array of column names to select. If empty, selects all columns.
     * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
     *
     * @example
     * const users = await db.table('users').select(['id', 'name']).get();
     * console.log(users); // [{ id: 1, name: 'John' }, { id: 2, name: 'Jane' }]
     */
    select(fields?: string[]): this;
    /**
     * Adds a WHERE condition to the query.
     *
     * @param {string} column - The column to filter by.
     * @param {string} operator - The comparison operator (e.g., '=', '>', '<').
     * @param {any} value - The value to compare against.
     * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
     *
     * @example
     * const users = await db.table('users').where('age', '>', 25).get();
     * console.log(users); // [{ id: 1, name: 'John', age: 30 }]
     */
    where(column: string, operator: string | undefined, value: any): this;
    /**
     * Adds an OR WHERE condition to the query.
     *
     * @param {string} column - The column to filter by.
     * @param {string} operator - The comparison operator (e.g., '=', '>', '<').
     * @param {any} value - The value to compare against.
     * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
     *
     * @example
     * const users = await db.table('users').where('age', '>', 25).orWhere('name', '=', 'Jane').get();
     * console.log(users); // [{ id: 1, name: 'John', age: 30 }, { id: 2, name: 'Jane', age: 25 }]
     */
    orWhere(column: string, operator: string | undefined, value: any): this;
    /**
     * Adds a grouped WHERE condition to the query.
     *
     * @param {Function} callback - A callback function that receives a new TableQuery instance to build the grouped conditions.
     * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
     *
     * @example
     * const users = await db.table('users').whereGroup(query => {
     *     query.where('age', '>', 25).orWhere('name', '=', 'Jane');
     * }).get();
     * console.log(users); // [{ id: 1, name: 'John', age: 30 }, { id: 2, name: 'Jane', age: 25 }]
     */
    whereGroup(callback: any): this;
    or(): this;
    and(): this;
    /**
     * Adds a WHERE BETWEEN condition to the query.
     *
     * @param {string} column - The column to filter by.
     * @param {Array<any>} values - An array with two values representing the range.
     * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
     *
     * @example
     * const users = await db.table('users').whereBetween('age', [20, 30]).get();
     * console.log(users); // [{ id: 1, name: 'John', age: 30 }, { id: 2, name: 'Jane', age: 25 }]
     */
    whereBetween(column: string, [value1, value2]: any): this;
    /**
     * Adds a WHERE IN condition to the query.
     *
     * @param {string} column - The column to filter by.
     * @param {Array<any>} values - An array of values to match.
     * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
     *
     * @example
     * const users = await db.table('users').whereIn('id', [1, 2]).get();
     * console.log(users); // [{ id: 1, name: 'John' }, { id: 2, name: 'Jane' }]
     */
    whereIn(column: string, values: any): this;
    /**
    * Adds a WHERE IS NULL condition to the query.
    *
    * @param {string} column - The column to filter by.
    * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
    *
    * @example
    * const users = await db.table('users').whereNull('email').get();
    * console.log(users); // [{ id: 3, name: 'Alice', email: null }]
    */
    whereNull(column: string): this;
    /**
     * Adds a WHERE IS NOT NULL condition to the query.
     *
     * @param {string} column - The column to filter by.
     * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
     *
     * @example
     * const users = await db.table('users').whereNotNull('email').get();
     * console.log(users); // [{ id: 1, name: 'John', email: 'john@example.com' }]
     */
    whereNotNull(column: string): this;
    buildQuery(includeSelect?: boolean): string;
    buildConditions(): string;
    /**
     * Adds a JOIN clause to the query.
     *
     * @param {string} table - The table to join.
     * @param {string} column1 - The column from the current table.
     * @param {string} operator - The comparison operator (e.g., '=', '>', '<').
     * @param {string} column2 - The column from the joined table.
     * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
     *
     * @example
     * const users = await db.table('users').join('orders', 'users.id', '=', 'orders.user_id').get();
     * console.log(users); // [{ id: 1, name: 'John', order_id: 101 }]
     */
    join(table: string, column1: string, operator: string, column2: string): this;
    /**
     * Adds a LEFT JOIN clause to the query.
     *
     * @param {string} table - The table to join.
     * @param {string} column1 - The column from the current table.
     * @param {string} operator - The comparison operator (e.g., '=', '>', '<').
     * @param {string} column2 - The column from the joined table.
     * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
     *
     * @example
     * const users = await db.table('users').leftJoin('orders', 'users.id', '=', 'orders.user_id').get();
     * console.log(users); // [{ id: 1, name: 'John', order_id: 101 }, { id: 2, name: 'Jane', order_id: null }]
     */
    leftJoin(table: string, column1: string, operator: string, column2: string): this;
    /**
     * Adds a RIGHT JOIN clause to the query.
     *
     * @param {string} table - The table to join.
     * @param {string} column1 - The column from the current table.
     * @param {string} operator - The comparison operator (e.g., '=', '>', '<').
     * @param {string} column2 - The column from the joined table.
     * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
     *
     * @example
     * const users = await db.table('users').rightJoin('orders', 'users.id', '=', 'orders.user_id').get();
     * console.log(users); // [{ id: 1, name: 'John', order_id: 101 }, { id: null, name: null, order_id: 102 }]
     */
    rightJoin(table: string, column1: string, operator: string, column2: string): this;
    /**
     * Adds an ORDER BY clause to the query.
     *
     * @param {string} column - The column to order by.
     * @param {string} direction - The sorting direction ('ASC' or 'DESC').
     * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
     *
     * @example
     * const users = await db.table('users').orderBy('name', 'ASC').get();
     * console.log(users); // [{ id: 2, name: 'Jane' }, { id: 1, name: 'John' }]
     */
    orderBy(column: string, direction?: string): this;
    /**
     * Adds a GROUP BY clause to the query.
     *
     * @param {string} column - The column to group by.
     * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
     *
     * @example
     * const users = await db.table('users').groupBy('age').get();
     * console.log(users); // [{ age: 30, count: 1 }, { age: 25, count: 1 }]
     */
    groupBy(column: string): this;
    /**
     * Adds a DISTINCT clause to the query.
     *
     * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
     *
     * @example
     * const users = await db.table('users').distinct().select(['name']).get();
     * console.log(users); // [{ name: 'John' }, { name: 'Jane' }]
     */
    distinct(): this;
    /**
     * Adds a COUNT clause to the query.
     *
     * @param {string} column - The column to count (default is '*').
     * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
     *
     * @example
     * const count = await db.table('users').count().first();
     * console.log(count); // { count: 2 }
     */
    count(column?: string): this;
    /**
     * Adds a SUM clause to the query.
     *
     * @param {string} column - The column to sum.
     * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
     *
     * @example
     * const totalAge = await db.table('users').sum('age').first();
     * console.log(totalAge); // { sum: 55 }
     */
    sum(column: string): this;
    /**
     * Adds an AVG clause to the query.
     *
     * @param {string} column - The column to calculate the average.
     * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
     *
     * @example
     * const avgAge = await db.table('users').avg('age').first();
     * console.log(avgAge); // { avg: 27.5 }
     */
    avg(column: string): this;
    /**
     * Adds a MAX clause to the query.
     *
     * @param {string} column - The column to find the maximum value.
     * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
     *
     * @example
     * const maxAge = await db.table('users').max('age').first();
     * console.log(maxAge); // { max: 30 }
     */
    max(column: string): this;
    /**
     * Adds a MIN clause to the query.
     *
     * @param {string} column - The column to find the minimum value.
     * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
     *
     * @example
     * const minAge = await db.table('users').min('age').first();
     * console.log(minAge); // { min: 25 }
     */
    min(column: string): this;
    /**
     * Adds a LIMIT clause to the query.
     *
     * @param {number} number - The maximum number of rows to return.
     * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
     *
     * @example
     * const users = await db.table('users').limit(1).get();
     * console.log(users); // [{ id: 1, name: 'John', age: 30 }]
     */
    limit(number: number): this;
    /**
     * Adds pagination to the query using LIMIT and OFFSET.
     *
     * @param {number} number - The page number (starting from 1).
     * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
     *
     * @example
     * const users = await db.table('users').limit(1).page(2).get();
     * console.log(users); // [{ id: 2, name: 'Jane', age: 25 }]
     */
    page(number: number): this;
    /**
    * Executes the query and returns all matching rows.
    *
    * @returns {Promise<Array<Object>>} - Returns an array of rows.
    *
    * @example
    * const users = await db.table('users').get();
    * console.log(users); // [{ id: 1, name: 'John' }, { id: 2, name: 'Jane' }]
    */
    get(): Promise<mysql2_promise.QueryResult>;
    /**
     * Executes the query and returns the first matching row.
     *
     * @returns {Promise<Object | null>} - Returns the first row or null if no rows match.
     *
     * @example
     * const user = await db.table('users').first();
     * console.log(user); // { id: 1, name: 'John' }
     */
    first(): Promise<any>;
    /**
     * Finds a row by a specific column value.
     *
     * @param {any} value - The value to search for.
     * @param {string} column - The column to search in (default is 'id').
     * @returns {Promise<Object | null>} - Returns the first matching row or null if no rows match.
     *
     * @example
     * const user = await db.table('users').find(1);
     * console.log(user); // { id: 1, name: 'John' }
     */
    find(value: any, column?: string): Promise<any>;
    /**
     * Inserts one or more rows into the table.
     *
     * @param {Array<Object>} data - An array of objects representing the rows to insert.
     * @returns {Promise<Array<Object>>} - Returns an array of the inserted rows.
     *
     * @example
     * const newUsers = await db.table('users').insert([
     *     { name: 'Alice', age: 28 },
     *     { name: 'Bob', age: 32 }
     * ]);
     * console.log(newUsers); // [{ id: 3, name: 'Alice', age: 28 }, { id: 4, name: 'Bob', age: 32 }]
     */
    insert(data: any): Promise<any>;
    /**
     * Inserts one or more rows into the table.
     *
     * @param {Array<Object>} data - An array of objects representing the rows to insert.
     * @returns {Promise<Array<Object>>} - Returns an array of the inserted rows.
     *
     * @example
     * const newUsers = await db.table('users').insert([
     *     { name: 'Alice', age: 28 },
     *     { name: 'Bob', age: 32 }
     * ]);
     * console.log(newUsers); // [{ id: 3, name: 'Alice', age: 28 }, { id: 4, name: 'Bob', age: 32 }]
     */
    update(data: any): Promise<mysql2_promise.QueryResult>;
    /**
     * Deletes rows from the table based on the defined conditions.
     *
     * @returns {Promise<Object>} - Returns the result of the delete operation.
     *
     * @example
     * const result = await db.table('users').where('id', '=', 1).delete();
     * console.log(result); // { affectedRows: 1 }
     */
    delete(): Promise<mysql2_promise.QueryResult>;
}
/**
 * Class to manage table columns, allowing adding, editing, or deleting columns.
 */
declare class Columns {
    #private;
    private conection;
    private tableName;
    constructor(tableName: string, conection?: Pool | null);
    /**
     * Retrieves the structure of the table columns.
     *
     * @returns {Promise<Object>} - Returns an object with the column details.
     *
     * @example
     * const columns = await db.table('users').columns().get();
     * console.log(columns); // { id: { type: 'INT', defaultValue: null, key: 'PRI', extra: 'auto_increment' } }
     */
    get(): Promise<any>;
    /**
     * Adds new columns to the table.
     *
     * @param {Array<Field>} fields - An array of objects defining the new columns.
     * @returns {Promise<boolean>} - Returns `true` if the operation was successful.
     *
     * @example
     * await db.table('users').columns().add([
     *     { name: 'email', type: 'VARCHAR', length: 255, defaultValue: 'example@example.com' }
     * ]);
     */
    add(fields: Field[]): Promise<boolean>;
    /**
     * Edits existing columns in the table.
     *
     * @param {Array<Field>} fields - An array of objects defining the columns to edit.
     * @returns {Promise<boolean>} - Returns `true` if the operation was successful.
     *
     * @example
     * await db.table('users').columns().edit([
     *     { name: 'email', type: 'VARCHAR', length: 255, defaultValue: 'new@example.com' }
     * ]);
     */
    edit(fields: Field[]): Promise<boolean>;
    /**
     * Deletes columns from the table.
     *
     * @param {Array<string>} fields - An array of column names to delete.
     * @returns {Promise<boolean>} - Returns `true` if the operation was successful.
     *
     * @example
     * await db.table('users').columns().delete(['email']);
     */
    delete(fields: string[]): Promise<boolean>;
}
declare const db: MySQL;

export { db as default };
